@@ -15,11 +15,11 @@ test_that("print method obfuscates Authorization header unless requested", {
   req <- request("https://example.com") %>%
     req_auth_basic("user", "SECRET")
   output <- testthat::capture_messages(print(req))
-  expect_false(any(grepl("SECRET", output)))
+  expect_false(any(grepl("SECRET", output, fixed = TRUE)))
 
   output <- testthat::capture_messages(print(req, redact_headers = FALSE))
-  expect_true(any(grepl("Authorization: 'Basic", output)))
-  expect_false(any(grepl("REDACTED", output)))
+  expect_true(any(grepl("Authorization: 'Basic", output, fixed = TRUE)))
+  expect_false(any(grepl("REDACTED", output, fixed = TRUE)))
 })
 
 test_that("check_request() gives useful error", {
